library IEEE;
use IEEE.std_logic_1164.all;

architecture TxUnit_arch of TxUnit is
    
    -- Le tampon et le registre utilises pour stocker 'data'
    signal bufferE, registerE : std_logic_vector(7 downto 0);
    -- Permet de tester l'etat du tampon et du registre
    signal bufEVar, regEVar : std_logic := '1';
    
    type t_etat is (initial, load, start, send, stop);
    signal etat : t_etat;
    
	 begin
	 bufE <= bufEVar;
	 regE <= regEVar;
    
    p_etat : process(clk, reset)
	 -- Compteur pour l'emission
	 variable cpt : integer range 0 to 9 := 0;
	 
	 begin
	 
	 if (reset = '0') then
		bufEVar <= '1';
		regEVar <= '1';
		bufferE <= (others => '0');
		registerE <= (others => '0');
		txd <= '1';
	elsif (clk = '1' and clk'event) then
	 
	 	--Chargement dans le buffer a chaque top d'horloge
		if(ld='1' and bufEvar='1') then
			bufEVar <= '0';
			bufferE <= data;
		end if;
	 
		-- Chargement dans le registre
	 	if(enable = '1' and bufEVar='0' and regEVar = '1') then
			bufEVar <= '1';
			regEVar <= '0';
			registerE <= bufferE;
		end if;
		
		case etat is
			when initial => if ((ld='1' and bufEVar='1') or (bufEVar='0')) then
										etat <= load;
									end if;
			when load =>	if (regEVar = '0') then
									etat <= start;
								end if;
			when start => 	if (enable = '1') then
									txd <= '0'; -- Envoi du bit de start
									cpt := 0;
									etat <= send;
								end if;
			when send => 	if (enable = '1') then
									if (cpt = 7) then
										regEVar <= '1';
										etat <= stop;
									end if;
									txd <= registerE(7 - cpt);
									cpt := cpt + 1;
								end if;
			when stop =>	if (enable = '1') then
									txd <= '1'; -- Envoi du bit de stop
									-- Si on a une autre donnee, on la charge directement
									if ((ld='1' and bufEVar='1') or (bufEVar='0')) then
										etat <= load;
									else
										etat <= initial;
									end if;
								end if;
			end case;
			
		end if;
		end process;
	 
end TxUnit_arch;
